#!/usr/bin/env python
# coding: utf-8

# # Chapter 2: Computing and Displaying Data

# ## Spherical astronomy

# ### Declination of the Sun

# In[ ]:


import math

N = 171 # day of 1st solstice
omega = 2*math.pi/365.24 # angular velocity in rad/day
ecl = math.radians(23.44) # obliquity of the ecliptic

# approximate expression for declination of the Sun
delta = -math.asin(math.sin(ecl)*math.cos(omega*(N+10)))
print("declination = {:.2f} deg".format(math.degrees(delta)))


# In[ ]:


import numpy as np

# equinoxes and solstices in 2020
N = np.array([79, 171, 265, 355])


# In[ ]:


print(N)
print(N.size)
print(N.dtype)


# In[ ]:


print(N[1])
print(N[-3])


# In[ ]:


delta = -np.arcsin(math.sin(ecl)*np.cos(omega*(N+10)))
print(np.degrees(delta))


# In[ ]:


# add 10 to each element of N
tmp = N+10
print(tmp)
print(tmp.dtype)

# multipy by omega
tmp = omega*tmp
print(tmp)
print(tmp.dtype)

# calculate the cosine of each element in the resulting array 
# and multipy by the sine of the obliquity
tmp = math.sin(ecl)*np.cos(tmp)
print(tmp)

# calculate negative arcsine of each element
delta = -np.arcsin(tmp)
print(np.degrees(delta))


# In[ ]:


delta = -np.asin(tmp) # arcsine in numpy has a different identifier than in math module!


# In[ ]:


print("declination = {:.2f} deg".      format(math.degrees(delta[1])))


# In[ ]:


for val in delta:
    print("declination = {:6.2f} deg".format(math.degrees(val)))


# In[ ]:


print("i  day  delta [deg]")
for i,val in enumerate(delta):
    print("{1:d}  {2:3d}  {0:8.2f}".format(math.degrees(val),i,N[i]))


# In[ ]:


print("day  delta [deg]")
for row in zip(N,delta):
    print("{0:3d}  {1:8.2f}".
          format(row[0],math.degrees(row[1])))


# Each row is a tuple:

# In[ ]:


for row in zip(N,delta):
    print(row)


# ### Diurnal arc

# Detailed documentation of coordinate systems in Astropy: [docs.astropy.org/en/stable/coordinates/](https://docs.astropy.org/en/stable/coordinates/)

# #### Betelgeuse

# In[ ]:


from astropy.coordinates import SkyCoord, EarthLocation

betelgeuse = SkyCoord.from_name('Betelgeuse')
print(betelgeuse)


# In[ ]:


delta = betelgeuse.dec
print(delta)


# In[ ]:


import astropy.units as u

# geographical position of the observer
obs = EarthLocation(lat=53*u.deg+28*u.arcmin+49*u.arcsec, 
                    lon=10*u.deg+14*u.arcmin+23*u.arcsec)
             
# get latitude    
phi = obs.lat


# In[ ]:


import math

h = math.acos(-math.tan(delta.radian) * 
              math.tan(phi.radian))


# In[ ]:


T = (math.degrees(2*h)/360)*u.sday
print("T = {:.2f}".format(T.to(u.h)))


# #### Annual variation of day length

# In[ ]:


import numpy as np

N = np.arange(365) # array with elements 0,1,2,...,364
omega = 2*math.pi/365.24 # Earth's angular velocity in rad/day
ecl = math.radians(23.44) # obliquity of the ecliptic

# calculate declination of the Sun for all days of the year
delta = -np.arcsin(math.sin(ecl)*np.cos(omega*(N+10)))


# In[ ]:


# calculate day length in solar hours
h = np.arccos(-np.tan(delta)*math.tan(phi.radian))
T = (np.degrees(2*h)/360) * u.sday.to(u.h) 


# In[ ]:


# inspect single element
i = 0
print("day {0:d}: declination = {1:.2f} deg, T = {2:.2f} h".      format(N[i],math.degrees(delta[i]),T[i]))


# In[ ]:


import matplotlib.pyplot as plt
get_ipython().run_line_magic('matplotlib', 'inline')

plt.plot(N, T)
plt.xlabel("Day")
plt.ylabel("Day length [hr]")
plt.savefig("daylength.pdf")


# In[ ]:


print("Minimum day length = {:5.2f} h".format(T.min()))
print("Maximum day length = {:5.2f} h".format(T.max()))


# In[ ]:


phi = math.radians(78+13/60) # latitude of Longyearbyen


# In[ ]:


# calculate day length in solar hours for all days of the year
h = np.arccos(-np.tan(delta)*math.tan(phi))
T = (np.degrees(2*h)/360)*u.sday.to(u.h)


# In[ ]:


tmp = np.clip(-np.tan(delta)*math.tan(phi), -1.0, 1.0)
h = np.arccos(tmp)
T = (np.degrees(2*h)/360)*u.sday.to(u.h)


# In[ ]:


plt.plot(N, T)
plt.xlabel("Day")
plt.ylabel("Day length [hr]")


# In[ ]:


phi = { 'Hamburg'      : obs.lat.radian,
        'Longyearbyen' : math.radians(78 + 13/60) }


# In[ ]:


print(phi['Hamburg'])


# In[ ]:


phi['New York'] = math.radians(40 + 43/60)
phi['Bangkok']  = math.radians(13 + 45/60)


# In[ ]:


print(len(phi))


# In[ ]:


for key in phi:
    print(key + ": {:.2f} deg".format(math.degrees(phi[key])))
    
    h = np.arccos(np.clip(-np.tan(delta)*math.tan(phi[key]),
                          -1.0, 1.0))
    T = (np.degrees(2*h)/360) * u.sday.to(u.h)
    
    plt.plot(N, T, label=key)
   
plt.xlabel("Day")
plt.xlim(0,364)
plt.ylabel("Day length [hr]")
plt.ylim(0,24)
plt.legend(loc='upper right')
plt.savefig("daylength.pdf")


# ### Observing Betelgeuse

# In[ ]:


import astropy.units as u
from astropy.coordinates import SkyCoord, EarthLocation,    AltAz, get_sun

# geographical position of the observer
obs = EarthLocation(lat=53*u.deg+28*u.arcmin+49*u.arcsec, 
                    lon=10*u.deg+14*u.arcmin+23*u.arcsec)
             
# get latitude    
phi = obs.lat


# In[ ]:


from astropy.time import Time

utc_shift = 2*u.hour  # CEST time zone (+2h)

noon_cest = Time("2020-07-31 12:00:00") - utc_shift
print(noon_cest)


# In[ ]:


import numpy as np

# time array covering next 24 hours in steps of 5 min
elapsed = np.arange(0, 24*60, 5)*u.min
time = noon_cest + elapsed

# sequence of horizontal frames
frame_local_24h = AltAz(obstime=time, location=obs)


# In[ ]:


type(np.arange(0, 24*60, 5))


# In[ ]:


type(elapsed)


# In[ ]:


# is instance of type
isinstance(elapsed, u.quantity.Quantity)


# In[ ]:


# is instance of base class
isinstance(elapsed, np.ndarray)


# In[ ]:


type(time)


# In[ ]:


isinstance(time, np.ndarray)


# In[ ]:


import astropy.utils.misc

isinstance(time, astropy.utils.misc.ShapedLikeNDArray)


# In[ ]:


# star we want to observe
betelgeuse = SkyCoord.from_name('Betelgeuse')

betelgeuse_local = betelgeuse.transform_to(frame_local_24h)


# In[ ]:


sun = get_sun(time)

sun_local = sun.transform_to(frame_local_24h)


# In[ ]:


import matplotlib.pyplot as plt
get_ipython().run_line_magic('matplotlib', 'inline')

elapsed_night = elapsed[np.where(sun_local.alt < 0)]
betelgeuse_night = betelgeuse_local.alt[np.where(sun_local.alt < 0)]

plt.plot(elapsed.to(u.h), sun_local.alt, 
         color='orange', label='Sun')
plt.plot(elapsed_night.to(u.h), betelgeuse_night, 
         color='red', label='Betelgeuse (night)')
plt.plot(elapsed.to(u.h), betelgeuse_local.alt, 
         color='red', linestyle=':', label='Betelgeuse (daylight)')

plt.xlabel('Time from noon [h]')
plt.xlim(0, 24)
plt.xticks(np.arange(13)*2)
plt.ylim(0, 60)
plt.ylabel('Altitude [deg]')
plt.legend(loc='upper center')
plt.savefig("Betelgeuse_obs_window.pdf")


# See [docs.astropy.org/en/stable/generated/examples/coordinates/plot_obs-planning.html](https://docs.astropy.org/en/stable/generated/examples/coordinates/plot_obs-planning.html#) for a similar example. 

# ## Kepler's laws of planetary motion

# In[ ]:


import math
import numpy as np
from scipy.constants import year,hour,au,G
from astropy.constants import M_sun

M = M_sun.value # mass of the Sun in kg

# orbital parameters of planets 
# see https://nssdc.gsfc.nasa.gov/planetary/factsheet/
# mass in kg
m = 1e24*np.array([0.33011, 4.8675, 5.9723, 0.64171,                   1898.19, 568.34, 86.813, 102.413])
# semi-major axis in m
a = 1e9*np.array([57.9, 108.21, 149.60, 227.92,                  778.57, 1433.53, 2872.46, 4495.06])
              
# use Kepler's third law to calculate period in s
T_test_mass = 2*math.pi * (G*M)**(-1/2) * a**(3/2)
T_two_body = 2*math.pi * (G*(M + m))**(-1/2) * a**(3/2)

print("T [yr]  dev [hr] dev rel.")
for val1,val2 in zip(T_test_mass,T_two_body):
    dev = val1 - val2
    if dev > hour:
        line = "{0:6.2f}  {1:<7.1f}  {2:.1e}"
    else:
        line = "{0:6.2f}  {1:7.4f}  {2:.1e}"
    print(line.format(val2/year, dev/hour, dev/val1))


# In[ ]:


import matplotlib.pyplot as plt
get_ipython().run_line_magic('matplotlib', 'inline')

plt.loglog(a/au, T_test_mass/year, 'blue', linestyle='--',           label='test mass')
plt.loglog(a/au, T_two_body/year, 'ro', label='planets')
plt.legend(loc='lower right')
plt.xlabel("semi-major axis [AU]")
plt.ylabel("orbital period [yr]")
plt.savefig("kepler_third_law.pdf")


# ## Gravitational and tidal forces

# In[ ]:


import numpy as np
from scipy.constants import g,G
from astropy.constants import R_earth,M_earth

M = 0.07346e24 # mass of the moon in kg
r = 3.844e8 # semi-major axis of moon orbit in m

coeff = G*M/r**3
accel_scale = 2*coeff*R_earth.value
print("tidal acceleration = {:.2e} m/s^2 = {:.2e} g".      format(accel_scale,accel_scale/g))

h = 15*M*R_earth.value**4/(8*M_earth.value*r**3)
print("size of tidal bulge = {:.2f} m".format(h))

# array of evenly spaced grid points along x- and y-axis
X = np.linspace(-1.1, 1.1, num=23, endpoint=True)
Y = np.linspace(-1.1, 1.1, num=23, endpoint=True)
print(X)

# create two-dimensional mesh grid scaled by Earth radius
R_x, R_y = np.meshgrid(R_earth.value*X, R_earth.value*Y)
print(R_x.shape)
print(R_x[11,21],R_y[11,21])

# radial distances of mesh points from (0,0)
R = np.sqrt(R_x*R_x + R_y*R_y)

# components of tidal acceleration field within Earth radius
accel_x = np.ma.masked_where(R > R_earth.value, 2*coeff*R_x)
accel_y = np.ma.masked_where(R > R_earth.value, -coeff*R_y)


# In[ ]:


accel_x.size


# ```R_x[11,21]``` and ```R_y[11,21]``` are given by the following to elements of ```X``` and ```Y```

# In[ ]:


X[21]*R_earth.value


# In[ ]:


Y[11]*R_earth.value


# In[ ]:


import matplotlib.pyplot as plt
from matplotlib.patches import Circle
get_ipython().run_line_magic('matplotlib', 'inline')

fig, ax = plt.subplots(figsize=(6,6),dpi=100)
ax.set_aspect('equal')

# plot vector field
arrows = ax.quiver(X, Y, accel_x, accel_y, color='blue')
ax.quiverkey(arrows, X=0.1, Y=0.95, U=accel_scale,
             label=r'$1.1\times 10^{-6}\;\mathrm{m/s}^2$', 
             labelpos='E')

# add a circle
circle = Circle((0, 0), 1, alpha=0.2, edgecolor=None)
ax.add_patch(circle)

ax.set_xlabel(r'$x/R_{\mathrm{E}}$', fontsize=12)
ax.set_ylabel(r'$y/R_{\mathrm{E}}$', fontsize=12)

plt.savefig("tidal_accel_earth.pdf")


# In[ ]:




